/*
** Copyright 2015 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

#include "com/centreon/broker/compression/factory.hh"
#include "com/centreon/broker/compression/internal.hh"
#include "com/centreon/broker/io/protocols.hh"

using namespace com::centreon::broker;

/**
 *  Register the compression protocol.
 */
void compression::load() {
  // Register compression layer.
  io::protocols::instance().reg(
                              "compression",
                              compression::factory(),
                              6,
                              6);
  return ;
}

/**
 *  Unregister the compression protocol.
 */
void compression::unload() {
  // Unregister compression layer.
  io::protocols::instance().unreg("compression");
  return ;
}
