/*
** Copyright 2009-2011,2015 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

#ifndef CCB_NEB_EVENTS_HH
#  define CCB_NEB_EVENTS_HH

#  include "com/centreon/broker/neb/acknowledgement.hh"
#  include "com/centreon/broker/neb/comment.hh"
#  include "com/centreon/broker/neb/custom_variable.hh"
#  include "com/centreon/broker/neb/custom_variable_status.hh"
#  include "com/centreon/broker/neb/downtime.hh"
#  include "com/centreon/broker/neb/event_handler.hh"
#  include "com/centreon/broker/neb/flapping_status.hh"
#  include "com/centreon/broker/neb/host.hh"
#  include "com/centreon/broker/neb/host_check.hh"
#  include "com/centreon/broker/neb/host_dependency.hh"
#  include "com/centreon/broker/neb/host_group.hh"
#  include "com/centreon/broker/neb/host_group_member.hh"
#  include "com/centreon/broker/neb/host_parent.hh"
#  include "com/centreon/broker/neb/host_status.hh"
#  include "com/centreon/broker/neb/instance.hh"
#  include "com/centreon/broker/neb/instance_configuration.hh"
#  include "com/centreon/broker/neb/instance_status.hh"
#  include "com/centreon/broker/neb/log_entry.hh"
#  include "com/centreon/broker/neb/module.hh"
#  include "com/centreon/broker/neb/service.hh"
#  include "com/centreon/broker/neb/service_check.hh"
#  include "com/centreon/broker/neb/service_dependency.hh"
#  include "com/centreon/broker/neb/service_group.hh"
#  include "com/centreon/broker/neb/service_group_member.hh"
#  include "com/centreon/broker/neb/service_status.hh"

#endif // CCB_NEB_EVENTS_HH
