/*
** Copyright 2009-2013 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

#ifndef CCB_TLS_CONNECTOR_HH
#  define CCB_TLS_CONNECTOR_HH

#  include <string>
#  include "com/centreon/broker/io/endpoint.hh"
#  include "com/centreon/broker/namespace.hh"

CCB_BEGIN()

namespace         tls {
  /**
   *  @class connector connector.hh "com/centreon/broker/tls/connector.hh"
   *  @brief Connect to a TLS peer.
   *
   *  Use a connected TLS stream to connect to a TLS peer.
   */
  class           connector : public io::endpoint {
  public:
                  connector(
                    std::string const& cert = std::string(),
                    std::string const& key = std::string(),
                    std::string const& ca = std::string());
                  connector(connector const& right);
                  ~connector();
    connector&    operator=(connector const& right);
    misc::shared_ptr<io::stream>
                  open();
    misc::shared_ptr<io::stream>
                  open(misc::shared_ptr<io::stream> lower);

  private:
    void          _internal_copy(connector const& right);

    std::string   _ca;
    std::string   _cert;
    std::string   _key;
  };
}

CCB_END()

#endif // !CCB_TLS_CONNECTOR_HH
