/*
** Copyright 2011-2013 Merethis
**
** This file is part of Centreon Engine.
**
** Centreon Engine is free software: you can redistribute it and/or
** modify it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Centreon Engine is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with Centreon Engine. If not, see
** <http://www.gnu.org/licenses/>.
*/

#ifndef CCE_DELETER_HH
#  define CCE_DELETER_HH

#  include "com/centreon/engine/deleter/command.hh"
#  include "com/centreon/engine/deleter/commandsmember.hh"
#  include "com/centreon/engine/deleter/comment.hh"
#  include "com/centreon/engine/deleter/contactgroup.hh"
#  include "com/centreon/engine/deleter/contactgroupsmember.hh"
#  include "com/centreon/engine/deleter/contact.hh"
#  include "com/centreon/engine/deleter/contactsmember.hh"
#  include "com/centreon/engine/deleter/customvariablesmember.hh"
#  include "com/centreon/engine/deleter/daterange.hh"
#  include "com/centreon/engine/deleter/downtime.hh"
#  include "com/centreon/engine/deleter/hostdependency.hh"
#  include "com/centreon/engine/deleter/hostescalation.hh"
#  include "com/centreon/engine/deleter/hostgroup.hh"
#  include "com/centreon/engine/deleter/host.hh"
#  include "com/centreon/engine/deleter/hostsmember.hh"
#  include "com/centreon/engine/deleter/listmember.hh"
#  include "com/centreon/engine/deleter/objectlist.hh"
#  include "com/centreon/engine/deleter/servicedependency.hh"
#  include "com/centreon/engine/deleter/serviceescalation.hh"
#  include "com/centreon/engine/deleter/servicegroup.hh"
#  include "com/centreon/engine/deleter/service.hh"
#  include "com/centreon/engine/deleter/servicesmember.hh"
#  include "com/centreon/engine/deleter/timeperiodexclusion.hh"
#  include "com/centreon/engine/deleter/timeperiod.hh"
#  include "com/centreon/engine/deleter/timerange.hh"

#endif // !CCE_DELETER_HH

