/*
** Copyright 2012 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

#include "com/centreon/broker/storage/remove_graph.hh"

using namespace com::centreon::broker;

/**
 *  Check that the remove_graph object properly default constructs.
 *
 *  @return 0 on success.
 */
int main() {
  // Build object.
  storage::remove_graph r;

  // Check properties values.
  return ((r.id != 0) || r.is_index);
}
