/*
** Copyright 2012-2013,2015 Centreon
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
**
** For more information : contact@centreon.com
*/

#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <unistd.h>
#include "com/centreon/broker/exceptions/msg.hh"
#include "com/centreon/broker/misc/misc.hh"

using namespace com::centreon::broker;

/**
 *  Get a temporary file path.
 *
 *  @return Temporary file path.
 */
std::string misc::temp_path() {
  char path[] = "/tmp/brokerXXXXXX";
  int retval(mkstemp(path));
  if (retval < 0) {
    char const* err_msg(strerror(errno));
    throw (exceptions::msg() << "cannot create temporary file: "
           << err_msg);
  }
  ::close(retval);
  ::remove(path);
  return (path);
}
